import emitter from 'tiny-emitter/instance';

export default class ClaraEvents {

  events = {
    ON_FIRST_CLARA_SCENE_INTERACTION: 'ON_FIRST_CLARA_SCENE_INTERACTION',
    ON_HOME_BUTTON_PRESSED: 'ON_HOME_BUTTON_PRESSED'
  };

  constructor(claraApi, sceneContainerId) {
    this.claraApi = claraApi;
    this.sceneContainerId = sceneContainerId;
    this.registerOnRendered(() => {

      const firstInteractionHandler = () => emitter.emit(this.events.ON_FIRST_CLARA_SCENE_INTERACTION);
      const sceneContainer = document.getElementById(this.sceneContainerId);

      sceneContainer.addEventListener('mousedown', firstInteractionHandler, { once: true });
      sceneContainer.addEventListener('touchstart', firstInteractionHandler, { once: true });

      const homeButton = document.querySelector('svg[title="Home"]');
      if (homeButton) {
        homeButton.addEventListener('click', () => {
          emitter.emit(this.events.ON_HOME_BUTTON_PRESSED);
        });
      }
    });
  }

  registerOnLoading = onLoading => {
    this.claraApi.on('loading', onLoading);
  };

  registerOnPreloaded = onPreLoaded => {
    this.claraApi.on('preloaded', onPreLoaded);
  };

  registerOnRendered = onRendered => {
    this.claraApi.on('rendered', onRendered);
  };

  registerOnFirstInteraction = onFirstInteraction => {
    emitter.on(this.events.ON_FIRST_CLARA_SCENE_INTERACTION, onFirstInteraction);
  };

  registerOnHomeButtonPressed = onHomeButtonPressed => {
    emitter.on(this.events.ON_HOME_BUTTON_PRESSED, onHomeButtonPressed);
  };
}
