import ClaraConstants from './clara-constants';

export default class ClaraScene {

  constructor(claraApi) {
    this.claraApi = claraApi;
  }

  findParent = nodeId => {
    return this.claraApi.scene.find({
      id: nodeId,
      parent: true
    });
  };

  findRootModelOrNull = () => {
    let rootNodeId = this.findFirstLevelNodeInScene(ClaraConstants.MODEL);

    if (!rootNodeId) {
      rootNodeId = this.findFirstLevelNodeInScene(ClaraConstants.NULL);
    }

    return rootNodeId;
  };

  findFirstLevelNodeInScene = type => {
    return this.claraApi.scene.find({ from: { name: 'Objects' }, shallow: true, type });
  };

  findMeshOrNullByName = name => {
    return this.findByNameAndTypes([ClaraConstants.POLYMESH, ClaraConstants.NULL, ClaraConstants.MODEL], name);
  };

  findByNameAndTypes = (types, name) => {
    return this.claraApi.scene.find({type: types, name});
  };

  findMeshesWithMaterialByName = materialName => {
    const materialPaths = [];
    const meshIds = this.claraApi.scene.filter({type: ClaraConstants.POLYMESH});
    meshIds.forEach(meshId => {
      const materialRef = this.getMaterialReference(meshId);
      if (materialRef !== null) {
        const existingMaterial = this.claraApi.scene.get({id: materialRef, property: 'name'});
        if (existingMaterial === materialName) {
          materialPaths.push(this.findMaterialReference(meshId));
        }
      }
    });
    return materialPaths;
  };

  findMaterialReference = meshId => {
    return this.claraApi.scene.find({id: meshId, plug: ClaraConstants.MATERIAL, property: ClaraConstants.REFERENCE});
  };

  findMaterial = name => {
    return this.findByNameAndTypes(ClaraConstants.MATERIAL, name);
  };

  findTexture = name => {
    return this.findByNameAndTypes(ClaraConstants.IMAGE, name);
  };

  getId = node => {
    return node._result; // eslint-disable-line no-underscore-dangle
  };

  getCameraTransform = () => {
    const cameraId = this.claraApi.player.getCamera();
    return this.getTransform(cameraId);
  };

  setTransform = (nodeId, position, rotation, scale) => {
    this.setPosition(nodeId, position);
    this.setRotation(nodeId, rotation);
    this.setScale(nodeId, scale);
  };

  setPosition = (nodeId, position) => {
    this.setTransformProperty(nodeId, position, ClaraConstants.TRANSLATION);
  };

  setRotation = (nodeId, rotation) => {
    this.setTransformProperty(nodeId, rotation, ClaraConstants.ROTATION);
  };

  setScale = (nodeId, scale) => {
    this.setTransformProperty(nodeId, scale, ClaraConstants.SCALE);
  };

  setTransformProperty = (nodeId, value, propertyName) => {
    this.claraApi.scene.set({
      id: nodeId,
      plug: ClaraConstants.TRANSFORM,
      property: propertyName
    }, value);
  };

  getTransform = nodeId => {
    const position = this.getPosition(nodeId);
    const rotation = this.getRotation(nodeId);
    return { position, rotation };
  };

  getPosition = nodeId => this.getTransformProperty(nodeId, ClaraConstants.TRANSLATION);

  getRotation = nodeId => this.getTransformProperty(nodeId, ClaraConstants.ROTATION);

  getScale = nodeId => this.getTransformProperty(nodeId, ClaraConstants.SCALE);

  getTransformProperty = (nodeId, propertyName) => this.claraApi.scene.get({
    id: nodeId,
    plug: ClaraConstants.TRANSFORM,
    property: propertyName
  });

  getMaterialReference = meshId => {
    return this.claraApi.scene.get({id: meshId, plug: ClaraConstants.MATERIAL, property: ClaraConstants.REFERENCE});
  };

  setMaterial = (meshId, materialId) => {
    const meshReference = {id: meshId, plug: ClaraConstants.MATERIAL, property: ClaraConstants.REFERENCE};
    this.claraApi.scene.set(meshReference, materialId);
  };

  createMaterial = (name, properties) => {
    const materialLibrary = this.claraApi.scene.find('%MaterialLibrary');
    const plugs = {
      Material: [
        [
          ClaraConstants.PHYSICAL,
          properties
        ]
      ]
    };
    return this.getId(this.claraApi.scene.addNode({
      name,
      type: ClaraConstants.MATERIAL,
      parent: materialLibrary,
      plugs
    }));
  };
}
